"use strict";
const ReParse = require('reparse').ReParse;
function formatAddressList(list) {
    if (!(list instanceof Array)) {
        return list;
    }
    return list.join(', ');
}
exports.formatAddressList = formatAddressList;
function readAddressList(obj) {
    if (!obj) {
        return [];
    }
    else if (obj instanceof Array) {
        return obj.reduce(function (result, item) {
            return result.concat(readAddressList(item));
        }, []);
    }
    return (new ReParse(obj)).start(addressList);
}
exports.readAddressList = readAddressList;
function addressList() {
    return this.sepEndBy(address, /^\s*,\s*/);
}
function address() {
    return this.choice(namedAddress, bareAddress);
}
function namedAddress() {
    return this.seq(phrase, /^\s*</m, bareAddress, /^>/)[3];
}
function bareAddress() {
    return this.seq(word, /^@/, word).slice(1).join('');
}
function phrase() {
    return this.many(word);
}
function word() {
    return this.skip(/^\s+/).choice(quoted, dottedAtom);
}
function quoted() {
    return this.match(/^"(?:\\.|[^"\r\n])+"/m);
}
function dottedAtom() {
    return this.match(/^[!#\$%&'\*\+\-\/\w=\?\^`\{\|\}~]+(?:\.[!#\$%&'\*\+\-\/\w=\?\^`\{\|\}~]+)*/m);
}
