"use strict";
const Util = require("./util");
const Address = require("./address");
class Message {
    constructor(headers) {
        this._headers = {};
        this._recipients = [];
        this._sender = null;
        this._body = '';
        if (headers) {
            for (let key in headers)
                this._headers[Util.titleCaseHeader(key)] = headers[key];
        }
        if (!('Date' in this._headers)) {
            this._headers['Date'] = Util.date(undefined);
        }
    }
    // headers
    get headers() {
        return this._headers;
    }
    // recipients
    get recipients() {
        this.update();
        return this._recipients;
    }
    // sender
    get sender() {
        this.update();
        return this._sender;
    }
    // body
    get body() {
        return this._body;
    }
    set body(body) {
        this._body = body || '';
    }
    Body(body) {
        this.body = body;
        return this;
    }
    update() {
        let list = Address.readAddressList(this._headers['Sender'] || this._headers['From']);
        this._sender = (list.length > 0) ? list[0] : null;
        let seen = {}, header;
        ['To', 'Cc', 'Bcc'].forEach((name) => {
            Address.readAddressList(this._headers[name]).forEach((mailbox) => {
                if (!(mailbox in seen)) {
                    seen[mailbox] = true;
                    this._recipients.push(mailbox);
                }
            });
        });
    }
    formatHeader(name, value) {
        return Util.foldHeader(name, Address.formatAddressList(value), undefined);
    }
    toString() {
        this.update();
        let headers = Util.extend({}, this.headers), result = [], value;
        // Put these headers in a particular order.
        Message.HEADERS.forEach((name) => {
            if (name in headers) {
                result.push(this.formatHeader(name, headers[name]));
                delete headers[name];
            }
        });
        // Hide Bcc recipients.
        if ('Bcc' in headers)
            delete headers['Bcc'];
        // Add the rest of the headers in no particular order.
        for (let key in headers)
            result.push(this.formatHeader(key, headers[key]));
        // The body is separated from the headers by an empty line.
        result.push('');
        result.push(Util.fill(this._body));
        return result.join('\r\n');
    }
}
Message.HEADERS = ['Date', 'Sender', 'From', 'To', 'Cc', 'Subject'];
exports.Message = Message;
